package com.docus.server.common.netty.server.handler;

import com.docus.core.util.StringUtils;
import com.docus.core.util.json.JSON;
import com.docus.server.common.netty.CommMsg;
import com.docus.server.common.netty.Payload;
import com.docus.server.common.netty.server.ChannelRepository;
import com.fasterxml.jackson.core.type.TypeReference;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.channel.group.ChannelGroup;
import io.netty.channel.group.DefaultChannelGroup;
import io.netty.util.concurrent.GlobalEventExecutor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.net.InetSocketAddress;

/**
 * 业务消息处理
 */
@Slf4j
@ChannelHandler.Sharable
@Component
public class NettyBusinessHandler extends SimpleChannelInboundHandler<ByteBuf> {

    private static final ChannelGroup GROUP = new DefaultChannelGroup(GlobalEventExecutor.INSTANCE);

    @Resource
    private ChannelRepository repository;

    protected void channelRead0(ChannelHandlerContext channelHandlerContext, Payload.Message message) {
        if (message.getCmd() == Payload.Message.type.AUTH) {
            log.info("接受到工控机<{}>的认证消息：{}", message.getClient(), message.getContent());
            InetSocketAddress ipSocket = (InetSocketAddress) channelHandlerContext.channel().remoteAddress();
            String clientIp = ipSocket.getAddress().getHostAddress();
            log.info("工控机:{}，连接上线,IP地址信息:{}", message.getClient(), clientIp);
            Channel channel = repository.get(message.getClient());
            if (channel != null && channel.isOpen()) {
                channel.close();
            }
            repository.put(message.getClient(), channelHandlerContext.channel());
        }
        if (message.getCmd() == Payload.Message.type.PRINT_DONE) {
            log.info("接受到打印回馈指令，内容{}", message.getContent());
        }
        if (message.getCmd() == Payload.Message.type.WHATEVER) {
            log.info("收到测试消息：{}", message.getContent());
        }
    }

    @Override
    protected void channelRead0(ChannelHandlerContext ctx, ByteBuf msg) throws Exception {
        ByteBuf buf = (ByteBuf) msg;
        //创建目标大小的数组
        byte[] barray = new byte[buf.readableBytes()];
        //把数据从bytebuf转移到byte[]
        buf.getBytes(0, barray);
        //将byte[]转成字符串用于打印
        String message = new String(barray);

        //空消息不处理
        if (!StringUtils.hasText(message)) {
            return;
        }

        CommMsg<Test> commMsg = JSON.fromJSONWithGeneric(message, new TypeReference<CommMsg<Test>>() {
        });

        String messageType = commMsg.getMessageType();

//        if (message.getCmd() == Payload.Message.type.AUTH) {
//            log.info("接受到工控机<{}>的认证消息：{}", message.getClient(), message.getContent());
//            InetSocketAddress ipSocket = (InetSocketAddress) channelHandlerContext.channel().remoteAddress();
//            String clientIp = ipSocket.getAddress().getHostAddress();
//            log.info("工控机:{}，连接上线,IP地址信息:{}", message.getClient(), clientIp);
//            Channel channel = repository.get(message.getClient());
//            if (channel != null && channel.isOpen()) {
//                channel.close();
//            }
//            repository.put(message.getClient(), channelHandlerContext.channel());
//        }
//        if (message.getCmd() == Payload.Message.type.PRINT_DONE) {
//            log.info("接受到打印回馈指令，内容{}", message.getContent());
//        }
//        if (message.getCmd() == Payload.Message.type.WHATEVER) {
//            log.info("收到测试消息：{}", message.getContent());
//        }

        //心跳
        //注册
        //业务消息
    }

    @Override
    public void channelActive(ChannelHandlerContext ctx) throws Exception {
        Channel channel = ctx.channel();

        GROUP.add(channel);
        System.out.println(channel.remoteAddress() + " 上线," + "在线数量：" + GROUP.size());
    }

    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        // 获取到当前要断开连接的Channel
        Channel channel = ctx.channel();
        System.out.println(channel.remoteAddress() + "下线，" + "在线数量：" + GROUP.size());
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        cause.printStackTrace();
        Channel channel = ctx.channel();
        if (channel.isActive()) {
            ctx.close();
        }
        super.exceptionCaught(ctx, cause);
    }

}
